@@ -9,23 +9,41 @@
 //!
 //! ## Usage
 //!
+//! Key generation, signing and verification can be implemented as follows:
 //! ```
+//! # #[cfg(feature="picnic")] {
 //! use picnic_bindings::{PicnicL1FSSigningKey, signature::{Signer, Verifier}};
 //!
 //! let (signing_key, verification_key) = PicnicL1FSSigningKey::random().expect("Key generation failed");
 //! let msg = "some message".as_bytes();
 //! let signature = signing_key.sign(msg);
 //! verification_key.verify(msg, &signature).expect("Verification failed");
+//! # }
+//! ```
+//!
+//! Keys and signatures support conversions to and from `&[u8]`. The following code example
+//! demonstrates the necessary steps for [SigningKey]:
+//! ```
+//! # #[cfg(feature="picnic")] {
+//! use picnic_bindings::{PicnicL1FSSigningKey};
+//! use std::convert::TryFrom;
+//!
+//! let (signing_key, verification_key) = PicnicL1FSSigningKey::random().expect("Key generation failed");
+//! let signing_key_2 = PicnicL1FSSigningKey::try_from(signing_key.as_ref()).expect("Deserialization failed");
+//! assert_eq!(signing_key, signing_key_2);
+//! # }
 //! ```
 //!
 //! Alternatively:
 //! ```
+//! # #[cfg(feature="picnic")] {
 //! use picnic_bindings::{DynamicSigningKey, PicnicL1FS, Parameters, signature::{Signer, Verifier}};
 //!
 //! let (signing_key, verification_key) = DynamicSigningKey::random(PicnicL1FS::PARAM).expect("Key generation failed");
 //! let msg = "some message".as_bytes();
 //! let signature = signing_key.sign(msg);
 //! verification_key.verify(msg, &signature).expect("Verification failed");
+//! # }
 //! ```
 
 #![warn(missing_docs)]
@@ -392,7 +410,7 @@ where
                 signature.resize(length, 0);
                 Ok(DynamicSignature { 0: signature })
             }
-            _ => Err(signature::Error::new()), // TODO: forward Error::from(ret)
+            _ => Err(signature::Error::new()),
         }
     }
 }
@@ -491,7 +509,7 @@ where
         };
         match ret {
             0 => Ok(()),
-            _ => Err(signature::Error::new()), // TODO: forward Error::from(ret)
+            _ => Err(signature::Error::new()),
         }
     }
 }
@@ -548,21 +566,19 @@ impl<P> Eq for VerificationKey<P> where P: Parameters {}
 #[derive(Clone, Debug)]
 pub struct DynamicSigningKey {
     data: picnic_privatekey_t,
-    params: picnic_params_t, // FIXME: this is already stored in data; re-use that
 }
 
 impl DynamicSigningKey {
-    fn new(params: picnic_params_t) -> Self {
+    fn new() -> Self {
         Self {
             data: default_picnic_privatekey_t(),
-            params,
         }
     }
 
     /// Sample a new random signing key and the corresponding verification key
     pub fn random(params: picnic_params_t) -> Result<(Self, DynamicVerificationKey), Error> {
-        let mut sk = DynamicSigningKey::new(params);
-        let mut vk = DynamicVerificationKey::new(params);
+        let mut sk = DynamicSigningKey::new();
+        let mut vk = DynamicVerificationKey::new();
 
         let ret = unsafe { picnic_keygen(params, &mut vk.data, &mut sk.data) };
         match ret {
@@ -571,9 +587,9 @@ impl DynamicSigningKey {
         }
     }
 
-    /// Get verification key
+    /// Return corresponding verification key
     pub fn verifying_key(&self) -> Result<DynamicVerificationKey, Error> {
-        let mut vk = DynamicVerificationKey::new(self.params);
+        let mut vk = DynamicVerificationKey::new();
 
         let ret = unsafe { picnic_sk_to_pk(&self.data, &mut vk.data) };
         match ret {
@@ -581,6 +597,10 @@ impl DynamicSigningKey {
             _ => Err(ret.into()),
         }
     }
+
+    fn param(&self) -> picnic_params_t {
+        unsafe { picnic_get_private_key_param(&self.data) }
+    }
 }
 
 impl Drop for DynamicSigningKey {
@@ -593,8 +613,9 @@ impl Drop for DynamicSigningKey {
 
 impl PartialEq for DynamicSigningKey {
     fn eq(&self, other: &Self) -> bool {
-        self.params == other.params && {
-            let size = unsafe { picnic_get_private_key_size(self.params) };
+        let self_param = self.param();
+        self_param == other.param() && {
+            let size = unsafe { picnic_get_private_key_size(self_param) };
             self.data.data[..size] == other.data.data[..size]
         }
     }
@@ -606,21 +627,23 @@ impl Eq for DynamicSigningKey {}
 #[derive(Clone, Debug)]
 pub struct DynamicVerificationKey {
     data: picnic_publickey_t,
-    params: picnic_params_t, // FIXME: this is already stored in data; re-use that
 }
 
 impl DynamicVerificationKey {
-    fn new(params: picnic_params_t) -> Self {
+    fn new() -> Self {
         Self {
             data: default_picnic_publickey_t(),
-            params,
         }
     }
+
+    fn param(&self) -> picnic_params_t {
+        unsafe { picnic_get_public_key_param(&self.data) }
+    }
 }
 
 impl signature::Signer<DynamicSignature> for DynamicSigningKey {
     fn try_sign(&self, msg: &[u8]) -> Result<DynamicSignature, signature::Error> {
-        let mut length = unsafe { picnic_signature_size(self.params) };
+        let mut length = unsafe { picnic_signature_size(self.param()) };
         let mut signature = vec![0; length];
 
         let ret = unsafe {
@@ -637,7 +660,7 @@ impl signature::Signer<DynamicSignature> for DynamicSigningKey {
                 signature.resize(length, 0);
                 Ok(DynamicSignature { 0: signature })
             }
-            _ => Err(signature::Error::new()), // TODO: forward Error::from(ret)
+            _ => Err(signature::Error::new()),
         }
     }
 }
@@ -655,15 +678,16 @@ impl signature::Verifier<DynamicSignature> for DynamicVerificationKey {
         };
         match ret {
             0 => Ok(()),
-            _ => Err(signature::Error::new()), // TODO: forward Error::from(ret)
+            _ => Err(signature::Error::new()),
         }
     }
 }
 
 impl PartialEq for DynamicVerificationKey {
     fn eq(&self, other: &Self) -> bool {
-        self.params == other.params && {
-            let size = unsafe { picnic_get_public_key_size(self.params) };
+        let self_param = self.param();
+        self_param == other.param() && {
+            let size = unsafe { picnic_get_public_key_size(self_param) };
             self.data.data[..size] == other.data.data[..size]
         }
     }
